from __future__ import annotations

from copy import deepcopy
from pathlib import Path

from gperm.adapters.base import AdapterContext, AdapterMetadata, InlineResult, claude_rule_lists, resolved_directories
from gperm.formats import read_data
from gperm.model import PermissionProfile
from gperm.operations import OperationPlan
from gperm.util import home_dir


def _permission_mode(profile: PermissionProfile) -> str:
    mapping = {
        "default": "default",
        "auto-edit": "acceptEdits",
        "full-auto": "dontAsk",
        "plan": "plan",
    }
    return mapping.get(profile.approval, "default")


class ClaudeAdapter:
    metadata = AdapterMetadata(
        key="claude",
        display_name="Claude Code",
        aliases=("claude",),
        default_command="claude",
        tested_version="2.1.62",
        global_support="native",
        project_support="native",
        inline_support="native",
        docs=(
            "https://code.claude.com/docs/en/settings",
            "https://code.claude.com/docs/en/permissions",
        ),
    )

    def _path(self, context: AdapterContext, *, project: bool) -> Path:
        if project:
            return context.project_root / ".claude" / "settings.json"
        return home_dir(context.env) / ".claude" / "settings.json"

    def _desired_managed(self, profile: PermissionProfile, context: AdapterContext) -> tuple[dict[str, object], list[str]]:
        rules = claude_rule_lists(profile)
        managed = {
            "defaultMode": _permission_mode(profile),
            "additionalDirectories": resolved_directories(profile.include_directories, context),
            "permissions": {
                "allow": rules["allow"],
                "ask": rules["ask"],
                "deny": rules["deny"],
            },
        }
        return managed, list(rules["warnings"])

    def _merge(self, current: dict[str, object], desired: dict[str, object]) -> dict[str, object]:
        merged = deepcopy(current)
        merged["defaultMode"] = desired["defaultMode"]
        merged["additionalDirectories"] = desired["additionalDirectories"]
        merged["permissions"] = desired["permissions"]
        return merged

    def build_operations(
        self,
        profile: PermissionProfile,
        context: AdapterContext,
        *,
        include_global: bool,
        include_project: bool,
    ) -> tuple[list[OperationPlan], list[str]]:
        operations: list[OperationPlan] = []
        warnings: list[str] = []
        desired_managed, desired_warnings = self._desired_managed(profile, context)
        warnings.extend(desired_warnings)

        if include_global:
            path = self._path(context, project=False)
            current = dict(read_data(path, "json"))
            managed = {
                "defaultMode": current.get("defaultMode"),
                "additionalDirectories": current.get("additionalDirectories", []),
                "permissions": current.get("permissions", {}),
            }
            operations.append(
                OperationPlan(
                    agent=self.metadata.key,
                    scope="global",
                    path=path,
                    file_format="json",
                    label="Claude global settings",
                    current_full=current,
                    desired_full=self._merge(current, desired_managed),
                    current_managed=managed,
                    desired_managed=desired_managed,
                )
            )

        if include_project:
            path = self._path(context, project=True)
            current = dict(read_data(path, "json"))
            managed = {
                "defaultMode": current.get("defaultMode"),
                "additionalDirectories": current.get("additionalDirectories", []),
                "permissions": current.get("permissions", {}),
            }
            operations.append(
                OperationPlan(
                    agent=self.metadata.key,
                    scope="project",
                    path=path,
                    file_format="json",
                    label="Claude project settings",
                    current_full=current,
                    desired_full=self._merge(current, desired_managed),
                    current_managed=managed,
                    desired_managed=desired_managed,
                )
            )

        return operations, warnings

    def inline_args(self, profile: PermissionProfile, context: AdapterContext) -> InlineResult:
        rules = claude_rule_lists(profile)
        args = ["--permission-mode", _permission_mode(profile)]
        if rules["allow"]:
            args.extend(["--allowedTools", *rules["allow"]])
        deny_rules = rules["deny"]
        if deny_rules:
            args.extend(["--disallowedTools", *deny_rules])
        for directory in resolved_directories(profile.include_directories, context):
            args.extend(["--add-dir", directory])
        warnings = list(rules["warnings"])
        if rules["ask"]:
            warnings.append("Claude inline flags cannot express ask-only rules; they are omitted from inline output.")
        return InlineResult(args=args, warnings=warnings)



from __future__ import annotations

from copy import deepcopy
from pathlib import Path

from gperm.adapters.base import AdapterContext, AdapterMetadata, InlineResult
from gperm.formats import read_data
from gperm.model import PermissionProfile
from gperm.operations import OperationPlan
from gperm.util import xdg_config_home


class AntigravityAdapter:
    metadata = AdapterMetadata(
        key="antigravity",
        display_name="Antigravity",
        aliases=("antigravity",),
        default_command="antigravity",
        tested_version="1.107.0",
        global_support="experimental",
        project_support="experimental",
        inline_support="none",
        docs=(),
        experimental=True,
    )

    def _path(self, context: AdapterContext, *, project: bool) -> Path:
        if project:
            return context.project_root / ".vscode" / "settings.json"
        return xdg_config_home(context.env) / "Antigravity" / "User" / "settings.json"

    def _managed(self, profile: PermissionProfile) -> tuple[dict[str, object], list[str]]:
        warnings: list[str] = [
            "Antigravity support is inferred from installed settings keys, not vendor documentation.",
        ]
        if profile.include_directories:
            warnings.append("Antigravity does not expose include-directory persistence in gperm 0.0.1.")
        if profile.allow_urls or profile.deny_urls:
            warnings.append("Antigravity URL permission persistence is not implemented in gperm 0.0.1.")
        if profile.approval == "auto-edit":
            warnings.append("Antigravity has no native auto-edit-only setting; gperm maps it conservatively.")

        terminal_auto = {command: True for command in profile.allow_shell}
        terminal_auto.update({command: False for command in profile.deny_shell})
        return {
            "chat.tools.autoApprove": profile.approval == "full-auto",
            "chat.tools.terminal.autoApprove": terminal_auto,
        }, warnings

    def build_operations(
        self,
        profile: PermissionProfile,
        context: AdapterContext,
        *,
        include_global: bool,
        include_project: bool,
    ) -> tuple[list[OperationPlan], list[str]]:
        operations: list[OperationPlan] = []
        managed, warnings = self._managed(profile)

        for project_flag in (False, True):
            if project_flag and not include_project:
                continue
            if not project_flag and not include_global:
                continue
            path = self._path(context, project=project_flag)
            current = dict(read_data(path, "jsonc"))
            desired_full = deepcopy(current)
            desired_full["chat.tools.autoApprove"] = managed["chat.tools.autoApprove"]
            desired_full["chat.tools.terminal.autoApprove"] = managed["chat.tools.terminal.autoApprove"]
            operations.append(
                OperationPlan(
                    agent=self.metadata.key,
                    scope="project" if project_flag else "global",
                    path=path,
                    file_format="jsonc",
                    label=f"Antigravity {'project' if project_flag else 'global'} settings",
                    current_full=current,
                    desired_full=desired_full,
                    current_managed={
                        "chat.tools.autoApprove": current.get("chat.tools.autoApprove"),
                        "chat.tools.terminal.autoApprove": current.get("chat.tools.terminal.autoApprove", {}),
                    },
                    desired_managed=managed,
                )
            )

        return operations, warnings

    def inline_args(self, profile: PermissionProfile, context: AdapterContext) -> InlineResult:
        return InlineResult(args=[], warnings=["Antigravity does not expose documented inline permission flags in gperm 0.0.1."])



from __future__ import annotations

from copy import deepcopy
from pathlib import Path

from gperm.adapters.base import AdapterContext, AdapterMetadata, InlineResult, copilot_permission_patterns, resolved_directories
from gperm.formats import read_data
from gperm.model import PermissionProfile
from gperm.operations import OperationPlan
from gperm.util import home_dir, uniq


class CopilotAdapter:
    metadata = AdapterMetadata(
        key="copilot",
        display_name="GitHub Copilot CLI",
        aliases=("copilot", "copilot-cli", "copilot cli"),
        default_command="copilot",
        tested_version="1.0.2",
        global_support="native",
        project_support="partial",
        inline_support="native",
        docs=(
            "https://docs.github.com/en/copilot/how-tos/copilot-cli/set-up-copilot-cli/configure-copilot-cli",
        ),
    )

    def _path(self, context: AdapterContext) -> Path:
        return home_dir(context.env) / ".copilot" / "config.json"

    def build_operations(
        self,
        profile: PermissionProfile,
        context: AdapterContext,
        *,
        include_global: bool,
        include_project: bool,
    ) -> tuple[list[OperationPlan], list[str]]:
        warnings: list[str] = []
        if profile.allow_tools or profile.deny_tools or profile.ask_tools or profile.allow_shell or profile.deny_shell or profile.ask_shell:
            warnings.append("Copilot persists folder and URL permissions, but tool-level rules are runtime-only via gperm inline/exec.")

        path = self._path(context)
        current = dict(read_data(path, "json"))
        desired_full = deepcopy(current)
        desired_managed: dict[str, object] = {}
        current_managed: dict[str, object] = {}

        if include_global:
            desired_full["allowed_urls"] = list(profile.allow_urls)
            desired_full["denied_urls"] = list(profile.deny_urls)
            desired_managed["allowed_urls"] = list(profile.allow_urls)
            desired_managed["denied_urls"] = list(profile.deny_urls)
            current_managed["allowed_urls"] = current.get("allowed_urls", [])
            current_managed["denied_urls"] = current.get("denied_urls", [])

        if include_project:
            trusted = [str(item) for item in current.get("trusted_folders", [])]
            target_entries = [str(context.project_root), *resolved_directories(profile.include_directories, context)]
            if profile.trust:
                trusted = uniq([*trusted, *target_entries])
            else:
                trusted = [item for item in trusted if item not in target_entries]
            desired_full["trusted_folders"] = trusted
            desired_managed["trusted_folders"] = [entry for entry in target_entries if entry in trusted]
            current_managed["trusted_folders"] = [entry for entry in target_entries if entry in current.get("trusted_folders", [])]

        if not desired_managed:
            return [], warnings

        return [
            OperationPlan(
                agent=self.metadata.key,
                scope="global/project" if include_global and include_project else ("project" if include_project else "global"),
                path=path,
                file_format="json",
                label="Copilot config",
                current_full=current,
                desired_full=desired_full,
                current_managed=current_managed,
                desired_managed=desired_managed,
            )
        ], warnings

    def inline_args(self, profile: PermissionProfile, context: AdapterContext) -> InlineResult:
        allow_patterns, deny_patterns, warnings = copilot_permission_patterns(profile)
        args: list[str] = []
        for pattern in allow_patterns:
            args.extend(["--allow-tool", pattern])
        for pattern in deny_patterns:
            args.extend(["--deny-tool", pattern])
        for url in profile.allow_urls:
            args.extend(["--allow-url", url])
        for url in profile.deny_urls:
            args.extend(["--deny-url", url])
        if profile.trust:
            args.extend(["--add-dir", str(context.project_root)])
        for directory in resolved_directories(profile.include_directories, context):
            args.extend(["--add-dir", directory])
        if profile.ask_tools or profile.ask_shell:
            warnings.append("Copilot inline flags cannot express ask-only rules; they are omitted from inline output.")
        return InlineResult(args=args, warnings=warnings)



from __future__ import annotations

from gperm.adapters.antigravity import AntigravityAdapter
from gperm.adapters.base import Adapter
from gperm.adapters.claude import ClaudeAdapter
from gperm.adapters.codex import CodexAdapter
from gperm.adapters.copilot import CopilotAdapter
from gperm.adapters.gemini import GeminiAdapter
from gperm.adapters.opencode import OpenCodeAdapter
from gperm.util import normalize_agent_name


ADAPTERS: dict[str, Adapter] = {}
for adapter in (
    ClaudeAdapter(),
    GeminiAdapter(),
    CopilotAdapter(),
    CodexAdapter(),
    OpenCodeAdapter(),
    AntigravityAdapter(),
):
    ADAPTERS[adapter.metadata.key] = adapter
    for alias in adapter.metadata.aliases:
        ADAPTERS[normalize_agent_name(alias)] = adapter


def get_adapter(name: str) -> Adapter:
    normalized = normalize_agent_name(name)
    if normalized not in ADAPTERS:
        raise KeyError(f"Unsupported agent: {name}")
    return ADAPTERS[normalized]


def unique_adapters(names: list[str]) -> list[Adapter]:
    result: list[Adapter] = []
    seen: set[str] = set()
    for name in names:
        adapter = get_adapter(name)
        if adapter.metadata.key not in seen:
            seen.add(adapter.metadata.key)
            result.append(adapter)
    return result



from __future__ import annotations

from pathlib import Path

from gperm.config import load_config


def test_load_config_prefers_project_override(tmp_path: Path) -> None:
    home = tmp_path / "home"
    user_config = home / ".config" / "gperm" / "config.toml"
    project = tmp_path / "project"
    project_config = project / ".gperm" / "config.toml"
    user_config.parent.mkdir(parents=True)
    project_config.parent.mkdir(parents=True)

    user_config.write_text(
        'version = 1\n'
        'default_profile = "balanced"\n'
        '[profiles.safe]\n'
        'approval = "plan"\n'
        'sandbox = "read-only"\n'
        '[profiles.balanced]\n'
        'approval = "default"\n'
        'sandbox = "workspace-write"\n'
        '[agents.codex]\n'
        'profile = "balanced"\n',
        encoding="utf-8",
    )
    project_config.write_text(
        '[project]\n'
        'profile = "safe"\n',
        encoding="utf-8",
    )

    env = {"HOME": str(home), "XDG_CONFIG_HOME": str(home / ".config")}
    loaded = load_config(project_root=project, env=env)

    assert [source.label for source in loaded.config.sources] == ["user config", "project override"]
    assert loaded.config.resolve_profile_name("codex", project, env) == "safe"



from __future__ import annotations

from dataclasses import dataclass, field
from pathlib import Path
from typing import Protocol

from gperm.model import PermissionProfile
from gperm.operations import OperationPlan
from gperm.util import expand_path, shell_command_variants, uniq


@dataclass(slots=True)
class AdapterContext:
    env: dict[str, str]
    project_root: Path
    user_gperm_dir: Path
    project_gperm_dir: Path


@dataclass(slots=True)
class InlineResult:
    args: list[str]
    warnings: list[str] = field(default_factory=list)


@dataclass(slots=True)
class AdapterMetadata:
    key: str
    display_name: str
    aliases: tuple[str, ...]
    default_command: str
    tested_version: str
    global_support: str
    project_support: str
    inline_support: str
    docs: tuple[str, ...] = ()
    experimental: bool = False


class Adapter(Protocol):
    metadata: AdapterMetadata

    def build_operations(
        self,
        profile: PermissionProfile,
        context: AdapterContext,
        *,
        include_global: bool,
        include_project: bool,
    ) -> tuple[list[OperationPlan], list[str]]: ...

    def inline_args(self, profile: PermissionProfile, context: AdapterContext) -> InlineResult: ...


GENERIC_CLAUDE_TOOLS = {
    "read": "Read",
    "edit": "Edit",
    "write": "Write",
    "shell": "Bash",
    "webfetch": "WebFetch",
}

GENERIC_OPENCODE_TOOLS = {
    "edit": "edit",
    "write": "edit",
    "webfetch": "webfetch",
}

GENERIC_COPILOT_TOOLS = {
    "edit": "write",
    "write": "write",
    "shell": "shell",
}


def resolved_directories(entries: list[str], context: AdapterContext) -> list[str]:
    return [str(expand_path(item, env=context.env, base=context.project_root)) for item in entries]


def default_decision(profile: PermissionProfile, tool: str) -> str:
    if profile.approval == "plan" or profile.sandbox == "read-only":
        return "deny" if tool in {"edit", "write", "shell", "webfetch"} else "ask"
    if tool in profile.deny_tools:
        return "deny"
    if tool in profile.ask_tools:
        return "ask"
    if tool in profile.allow_tools:
        return "allow"
    if profile.approval == "full-auto":
        return "allow"
    if profile.approval == "auto-edit" and tool in {"edit", "write"}:
        return "allow"
    return "ask"


def claude_rule_lists(profile: PermissionProfile) -> dict[str, list[str]]:
    allow: list[str] = []
    ask: list[str] = []
    deny: list[str] = []
    warnings: list[str] = []

    for tool in profile.allow_tools:
        mapped = GENERIC_CLAUDE_TOOLS.get(tool)
        if mapped:
            allow.append(mapped)
        else:
            warnings.append(f"Claude does not have a generic mapping for tool '{tool}'.")
    for tool in profile.ask_tools:
        mapped = GENERIC_CLAUDE_TOOLS.get(tool)
        if mapped:
            ask.append(mapped)
        else:
            warnings.append(f"Claude does not have a generic mapping for tool '{tool}'.")
    for tool in profile.deny_tools:
        mapped = GENERIC_CLAUDE_TOOLS.get(tool)
        if mapped:
            deny.append(mapped)
        else:
            warnings.append(f"Claude does not have a generic mapping for tool '{tool}'.")

    for command in profile.allow_shell:
        allow.extend(f"Bash({variant})" for variant in shell_command_variants(command))
    for command in profile.ask_shell:
        ask.extend(f"Bash({variant})" for variant in shell_command_variants(command))
    for command in profile.deny_shell:
        deny.extend(f"Bash({variant})" for variant in shell_command_variants(command))

    return {"allow": uniq(allow), "ask": uniq(ask), "deny": uniq(deny), "warnings": warnings}


def copilot_permission_patterns(profile: PermissionProfile) -> tuple[list[str], list[str], list[str]]:
    allow: list[str] = []
    deny: list[str] = []
    warnings: list[str] = []
    for tool in profile.allow_tools:
        mapped = GENERIC_COPILOT_TOOLS.get(tool)
        if mapped:
            allow.append(mapped)
        else:
            warnings.append(f"Copilot CLI cannot map generic tool '{tool}' to a native allow/deny pattern.")
    for tool in profile.deny_tools:
        mapped = GENERIC_COPILOT_TOOLS.get(tool)
        if mapped:
            deny.append(mapped)
        else:
            warnings.append(f"Copilot CLI cannot map generic tool '{tool}' to a native allow/deny pattern.")
    for command in profile.allow_shell:
        for variant in shell_command_variants(command):
            allow.append(f"shell({variant})")
    for command in profile.deny_shell:
        for variant in shell_command_variants(command):
            deny.append(f"shell({variant})")
    return uniq(allow), uniq(deny), warnings


def opencode_bash_rules(profile: PermissionProfile) -> str | dict[str, str]:
    if not profile.allow_shell and not profile.ask_shell and not profile.deny_shell:
        return default_decision(profile, "shell")
    rules: dict[str, str] = {"*": default_decision(profile, "shell")}
    for command in profile.allow_shell:
        rules[f"{command.strip()}*"] = "allow"
    for command in profile.ask_shell:
        rules[f"{command.strip()}*"] = "ask"
    for command in profile.deny_shell:
        rules[f"{command.strip()}*"] = "deny"
    return rules



from __future__ import annotations

import json
import os
import shlex
import shutil
import subprocess
from pathlib import Path
from typing import Annotated

import typer
from rich.console import Console
from rich.table import Table

from gperm import __version__
from gperm.adapters import get_adapter, unique_adapters
from gperm.adapters.base import AdapterContext
from gperm.config import load_config, render_config_for_display, write_sample_config
from gperm.formats import write_data
from gperm.i18n import Translator
from gperm.importers import build_import_plan, merged_import_config
from gperm.operations import OperationPlan
from gperm.util import xdg_config_home


env_snapshot = dict(os.environ)
translator = Translator.from_env(env_snapshot)
t = translator.text
console = Console()

app = typer.Typer(
    help=t("app.help"),
    no_args_is_help=True,
    add_completion=False,
    context_settings={"help_option_names": ["-h", "--help"]},
)
config_app = typer.Typer(
    help=t("config_app.help"),
    no_args_is_help=True,
    add_completion=False,
    context_settings={"help_option_names": ["-h", "--help"]},
)
app.add_typer(config_app, name="config")


def _version_callback(value: bool) -> None:
    if value:
        console.print(f"gperm {__version__}")
        raise typer.Exit()


ProjectOption = Annotated[
    Path | None,
    typer.Option("--project", help=t("project.help")),
]
ConfigOption = Annotated[
    Path | None,
    typer.Option("--config", help=t("config.help")),
]
AgentOption = Annotated[
    list[str] | None,
    typer.Option("--agent", "-a", help=t("agent.help")),
]
LevelOption = Annotated[
    str,
    typer.Option("--level", help=t("level.help"), case_sensitive=False, show_default=True),
]
ProfileOption = Annotated[
    str | None,
    typer.Option("--profile", help=t("profile.help")),
]
CommandOption = Annotated[
    str | None,
    typer.Option("--command", help=t("command.help")),
]

def _resolve_runtime(project: Path | None, config_path: Path | None) -> tuple[dict[str, str], object, AdapterContext]:
    env = dict(os.environ)
    project_root = (project or Path.cwd()).resolve()
    explicit = config_path.resolve() if config_path else None
    loaded = load_config(project_root=project_root, env=env, explicit_config=explicit)
    context = AdapterContext(
        env=env,
        project_root=project_root,
        user_gperm_dir=loaded.user_config_dir,
        project_gperm_dir=loaded.project_config_dir,
    )
    return env, loaded, context


def _print_sources(loaded: object) -> None:
    table = Table(title=t("config.active_sources"))
    table.add_column("Kind")
    table.add_column("Path")
    for source in loaded.config.sources:
        table.add_row(source.label, str(source.path) if source.path else "-")
    console.print(table)


def _selected_adapters(loaded: object, requested_agents: list[str] | None):
    if requested_agents:
        return unique_adapters(requested_agents)
    return unique_adapters(loaded.config.enabled_agents())


def _collect_operations(
    *,
    loaded: object,
    context: AdapterContext,
    requested_agents: list[str] | None,
    level: str,
    profile_override: str | None,
) -> tuple[list[OperationPlan], list[str], dict[str, str], bool]:
    include_global = level in {"global", "all"}
    include_project = level in {"project", "all"}
    warnings: list[str] = []
    resolved_profiles: dict[str, str] = {}
    ignored = include_project and loaded.config.is_ignored(context.project_root, context.env)
    if ignored:
        warnings.append(t("ignored.project"))

    operations: list[OperationPlan] = []
    for adapter in _selected_adapters(loaded, requested_agents):
        if ignored and include_project and not include_global:
            continue

        if profile_override:
            if profile_override not in loaded.config.profiles:
                raise typer.BadParameter(f"Unknown profile: {profile_override}")
            profile = loaded.config.profiles[profile_override]
        else:
            profile = loaded.config.resolve_profile(adapter.metadata.key, context.project_root, context.env)

        resolved_profiles[adapter.metadata.key] = profile.name
        ops, adapter_warnings = adapter.build_operations(
            profile,
            context,
            include_global=include_global,
            include_project=include_project and not ignored,
        )
        operations.extend(ops)
        warnings.extend(f"{adapter.metadata.display_name}: {warning}" for warning in adapter_warnings)

    return operations, warnings, resolved_profiles, ignored


def _status_rows(operations: list[OperationPlan]) -> list[dict[str, str]]:
    rows: list[dict[str, str]] = []
    for operation in operations:
        rows.append(
            {
                "agent": operation.agent,
                "scope": operation.scope,
                "status": t("status.drift") if operation.changed() else t("status.match"),
                "path": str(operation.path),
                "label": operation.label,
            }
        )
    return rows


def _print_status_table(rows: list[dict[str, str]], title: str) -> None:
    table = Table(title=title)
    table.add_column("Agent")
    table.add_column("Scope")
    table.add_column("Status")
    table.add_column("Path")
    for row in rows:
        table.add_row(row["agent"], row["scope"], row["status"], row["path"])
    console.print(table)


def _print_warnings(warnings: list[str]) -> None:
    if not warnings:
        return
    table = Table(title=t("warn.header"))
    table.add_column("Message")
    for warning in warnings:
        table.add_row(warning)
    console.print(table)


def _run_version(command: str) -> str:
    try:
        parts = shlex.split(command)
        completed = subprocess.run(
            [*parts, "--version"],
            check=False,
            capture_output=True,
            text=True,
            timeout=5,
        )
    except Exception:
        return ""

    output = (completed.stdout or completed.stderr).strip().splitlines()
    return output[0].strip() if output else ""


def _validated_level(level: str) -> str:
    normalized = level.lower()
    if normalized not in {"global", "project", "all"}:
        raise typer.BadParameter("Level must be one of: global, project, all.")
    return normalized


def _validated_format(output_format: str) -> str:
    normalized = output_format.lower()
    if normalized not in {"table", "json"}:
        raise typer.BadParameter("Format must be one of: table, json.")
    return normalized


def _profile_for(adapter, loaded, context: AdapterContext, profile_name: str | None):
    if profile_name:
        if profile_name not in loaded.config.profiles:
            raise typer.BadParameter(f"Unknown profile: {profile_name}")
        return loaded.config.profiles[profile_name]
    return loaded.config.resolve_profile(adapter.metadata.key, context.project_root, context.env)


@app.callback()
def root_callback(
    version: Annotated[
        bool | None,
        typer.Option("--version", callback=_version_callback, is_eager=True, help=t("version.help")),
    ] = None,
) -> None:
    _ = version


@config_app.command("show", help=t("config.show.help"))
def config_show(
    project: ProjectOption = None,
    config_path: ConfigOption = None,
    output_format: Annotated[str, typer.Option("--format", help=t("format.help"))] = "table",
) -> None:
    env, loaded, _ = _resolve_runtime(project, config_path)
    payload = render_config_for_display(loaded.config, loaded.project_root, env)
    output_format = _validated_format(output_format)

    if output_format == "json":
        console.print_json(json.dumps(payload, ensure_ascii=False))
        return

    _print_sources(loaded)
    console.print(f"{t('config.project_root')}: {loaded.project_root}")
    console.print(f"{t('config.default_profile')}: {loaded.config.default_profile}")
    console.print(f"{t('config.ignored')}: {payload['ignored']}")
    resolved_table = Table(title=t("config.resolved_profiles"))
    resolved_table.add_column("Agent")
    resolved_table.add_column("Profile")
    for agent, profile_name in payload["resolved_profiles"].items():
        resolved_table.add_row(agent, str(profile_name))
    console.print(resolved_table)


@config_app.command("init", help=t("config.init.help"))
def config_init(
    project: ProjectOption = None,
    project_local: Annotated[bool, typer.Option("--project-local", help="Write ./.gperm/config.toml.")] = False,
    force: Annotated[bool, typer.Option("--force", help="Overwrite an existing config file.")] = False,
    if_missing: Annotated[bool, typer.Option("--if-missing", help="Do nothing when the target config already exists.")] = False,
) -> None:
    env = dict(os.environ)
    project_root = (project or Path.cwd()).resolve()
    target = (project_root / ".gperm" / "config.toml") if project_local else (xdg_config_home(env) / "gperm" / "config.toml")
    if target.exists() and if_missing and not force:
        console.print(f"{t('init.skipped')}: {target}")
        return
    try:
        write_sample_config(target, force=force)
    except FileExistsError:
        console.print(t("init.exists"))
        raise typer.Exit(1)
    console.print(f"{t('init.wrote')}: {target}")


@app.command("agents", help=t("agents.help"))
def agents_command() -> None:
    from gperm.adapters import ADAPTERS

    seen: set[str] = set()
    table = Table(title="Supported agents")
    table.add_column("Agent")
    table.add_column("Aliases")
    table.add_column("Global")
    table.add_column("Project")
    table.add_column("Inline")
    table.add_column("Tested")
    table.add_column("Notes")
    for adapter in ADAPTERS.values():
        if adapter.metadata.key in seen:
            continue
        seen.add(adapter.metadata.key)
        notes = "experimental" if adapter.metadata.experimental else ""
        table.add_row(
            adapter.metadata.display_name,
            ", ".join(adapter.metadata.aliases),
            adapter.metadata.global_support,
            adapter.metadata.project_support,
            adapter.metadata.inline_support,
            adapter.metadata.tested_version,
            notes,
        )
    console.print(table)


@app.command("import", help=t("import.help"))
def import_command(
    agent_name: Annotated[str, typer.Argument()],
    source_path: Annotated[Path, typer.Argument()],
    target: Annotated[Path | None, typer.Option("--target", help="Target gperm config path.")] = None,
    profile: Annotated[str | None, typer.Option("--profile", help="Profile name to create or replace.")] = None,
    force: Annotated[bool, typer.Option("--force", help="Replace an existing imported profile with the same name.")] = False,
) -> None:
    env = dict(os.environ)
    normalized = agent_name.lower().replace(" ", "").replace("-", "")
    if normalized not in {"claude", "opencode"}:
        raise typer.BadParameter("Import currently supports only: claude, opencode.")

    resolved_source = source_path.expanduser().resolve()
    if not resolved_source.exists():
        raise typer.BadParameter(f"Source file does not exist: {resolved_source}")

    if normalized == "claude" and resolved_source.suffix != ".json":
        raise typer.BadParameter("Claude import expects a JSON settings file.")
    if normalized == "opencode" and resolved_source.suffix not in {".json", ".jsonc"}:
        raise typer.BadParameter("OpenCode import expects a JSON or JSONC config file.")

    plan = build_import_plan(
        "claude" if normalized == "claude" else "opencode",
        resolved_source,
        env,
        target_path=target,
        profile_name=profile,
    )
    merged = merged_import_config(plan.target_path, plan, replace_existing_profile=force)
    write_data(plan.target_path, "toml", merged)

    table = Table(title=f"{agent_name} import")
    table.add_column(t("doctor.item"))
    table.add_column(t("doctor.details"))
    table.add_row(t("import.source"), str(plan.source_path))
    table.add_row(t("import.target"), str(plan.target_path))
    table.add_row(t("import.scope"), plan.scope)
    table.add_row(t("import.profile"), plan.profile_name)
    console.print(table)
    _print_warnings(plan.warnings)
    console.print(f"{t('import.wrote')}: {plan.target_path}")


@app.command("doctor", help=t("doctor.help"))
def doctor_command(
    project: ProjectOption = None,
    config_path: ConfigOption = None,
    agent: AgentOption = None,
    output_format: Annotated[str, typer.Option("--format", help=t("format.help"))] = "table",
) -> None:
    env, loaded, context = _resolve_runtime(project, config_path)
    output_format = _validated_format(output_format)
    selected = _selected_adapters(loaded, agent)
    operations, warnings, resolved_profiles, ignored = _collect_operations(
        loaded=loaded,
        context=context,
        requested_agents=[adapter.metadata.key for adapter in selected],
        level="all",
        profile_override=None,
    )
    operations_by_agent: dict[str, list[OperationPlan]] = {}
    for operation in operations:
        operations_by_agent.setdefault(operation.agent, []).append(operation)

    failures = 0
    summary_rows: list[dict[str, str]] = []
    config_rows: list[dict[str, str]] = []
    agent_rows: list[dict[str, str]] = []

    active_sources = ", ".join(source.label for source in loaded.config.sources)
    config_rows.append(
        {
            "item": t("doctor.active_sources"),
            "status": t("doctor.ok") if loaded.config.sources else t("doctor.warn"),
            "details": active_sources or t("doctor.using_defaults"),
        }
    )
    config_rows.append(
        {
            "item": str(loaded.user_config_path),
            "status": t("doctor.found") if loaded.user_config_path.exists() else t("doctor.missing"),
            "details": "user config",
        }
    )
    config_rows.append(
        {
            "item": str(loaded.project_config_path),
            "status": t("doctor.found") if loaded.project_config_path.exists() else t("doctor.missing"),
            "details": "project override",
        }
    )
    if ignored:
        config_rows.append(
            {
                "item": t("doctor.project_ignored"),
                "status": t("doctor.warn"),
                "details": str(context.project_root),
            }
        )

    for adapter in selected:
        command = loaded.config.resolve_command(adapter.metadata.key)
        executable = shlex.split(command)[0]
        resolved_binary = shutil.which(executable, path=env.get("PATH"))
        version = _run_version(command) if resolved_binary else ""
        profile_name = resolved_profiles.get(adapter.metadata.key, loaded.config.resolve_profile(adapter.metadata.key, context.project_root, context.env).name)
        adapter_operations = operations_by_agent.get(adapter.metadata.key, [])
        drift_count = sum(1 for operation in adapter_operations if operation.changed())
        missing_paths = [str(operation.path) for operation in adapter_operations if not operation.path.exists()]
        status = t("doctor.ok")
        details: list[str] = [f"{t('doctor.profile')}: {profile_name}", f"{t('doctor.command')}: {command}"]

        if not resolved_binary:
            status = t("doctor.fail")
            failures += 1
            details.append("binary not found on PATH")
        else:
            details.append(resolved_binary)
            if version:
                details.append(version)

        if drift_count:
            if status != t("doctor.fail"):
                status = t("doctor.warn")
            details.append(f"drift={drift_count}")
        if missing_paths:
            if status != t("doctor.fail"):
                status = t("doctor.warn")
            details.append("missing: " + ", ".join(missing_paths))

        for operation in adapter_operations:
            op_status = t("doctor.ok")
            if not operation.path.exists():
                op_status = t("doctor.missing")
            elif operation.changed():
                op_status = t("doctor.warn")
            agent_rows.append(
                {
                    "agent": adapter.metadata.display_name,
                    "item": operation.scope,
                    "status": op_status,
                    "details": str(operation.path),
                }
            )

        summary_rows.append(
            {
                "item": adapter.metadata.display_name,
                "status": status,
                "details": " | ".join(details),
            }
        )

    payload = {
        "project_root": str(context.project_root),
        "ignored": ignored,
        "config": config_rows,
        "summary": summary_rows,
        "agent_paths": agent_rows,
        "warnings": warnings,
        "failures": failures,
    }

    if output_format == "json":
        console.print_json(json.dumps(payload, ensure_ascii=False))
    else:
        console.print(f"\n[bold cyan]gperm doctor[/bold cyan] - {t('doctor.title')}\n")

        config_table = Table(title=t("doctor.config"))
        config_table.add_column(t("doctor.item"))
        config_table.add_column(t("doctor.status"))
        config_table.add_column(t("doctor.details"))
        for row in config_rows:
            config_table.add_row(row["item"], row["status"], row["details"])
        console.print(config_table)

        summary_table = Table(title=t("doctor.summary"))
        summary_table.add_column(t("doctor.item"))
        summary_table.add_column(t("doctor.status"))
        summary_table.add_column(t("doctor.details"))
        for row in summary_rows:
            summary_table.add_row(row["item"], row["status"], row["details"])
        console.print(summary_table)

        paths_table = Table(title=t("doctor.agents"))
        paths_table.add_column("Agent")
        paths_table.add_column(t("doctor.item"))
        paths_table.add_column(t("doctor.status"))
        paths_table.add_column(t("doctor.details"))
        for row in agent_rows:
            paths_table.add_row(row["agent"], row["item"], row["status"], row["details"])
        console.print(paths_table)
        _print_warnings(warnings)
        if failures == 0:
            console.print(t("doctor.no_issues"))

    if failures:
        raise typer.Exit(1)


@app.command("check", help=t("check.help"))
def check_command(
    project: ProjectOption = None,
    config_path: ConfigOption = None,
    agent: AgentOption = None,
    level: LevelOption = "all",
    profile: ProfileOption = None,
    output_format: Annotated[str, typer.Option("--format", help=t("format.help"))] = "table",
) -> None:
    env, loaded, context = _resolve_runtime(project, config_path)
    output_format = _validated_format(output_format)
    operations, warnings, resolved_profiles, ignored = _collect_operations(
        loaded=loaded,
        context=context,
        requested_agents=agent,
        level=_validated_level(level),
        profile_override=profile,
    )
    rows = _status_rows(operations)
    payload = {
        "project_root": str(context.project_root),
        "ignored": ignored,
        "resolved_profiles": resolved_profiles,
        "results": rows,
        "warnings": warnings,
    }

    if output_format == "json":
        console.print_json(json.dumps(payload, ensure_ascii=False))
    else:
        _print_sources(loaded)
        _print_status_table(rows, t("check.summary"))
        _print_warnings(warnings)
        for operation in operations:
            if operation.changed():
                console.print(operation.diff_text())

    if any(operation.changed() for operation in operations):
        raise typer.Exit(1)


@app.command("sync", help=t("sync.help"))
def sync_command(
    project: ProjectOption = None,
    config_path: ConfigOption = None,
    agent: AgentOption = None,
    level: LevelOption = "all",
    profile: ProfileOption = None,
    yes: Annotated[bool, typer.Option("--yes", help=t("yes.help"))] = False,
    dry_run: Annotated[bool, typer.Option("--dry-run", help=t("dry_run.help"))] = False,
) -> None:
    _, loaded, context = _resolve_runtime(project, config_path)
    operations, warnings, _, ignored = _collect_operations(
        loaded=loaded,
        context=context,
        requested_agents=agent,
        level=_validated_level(level),
        profile_override=profile,
    )
    changed = [operation for operation in operations if operation.changed()]

    _print_sources(loaded)
    if ignored and not changed:
        _print_warnings(warnings)
        return

    if not changed:
        console.print(t("sync.no_changes"))
        _print_warnings(warnings)
        return

    _print_status_table(_status_rows(changed), t("sync.pending_changes"))
    for operation in changed:
        console.print(operation.diff_text())
    _print_warnings(warnings)

    if dry_run:
        return

    if not yes and not typer.confirm(t("sync.confirm")):
        console.print(t("sync.aborted"))
        raise typer.Exit(1)

    for operation in changed:
        operation.apply()
        console.print(f"{t('sync.applied')}: {operation.path}")


@app.command("inline", help=t("inline.help"), context_settings={"allow_extra_args": True, "ignore_unknown_options": True})
def inline_command(
    ctx: typer.Context,
    agent_name: Annotated[str, typer.Argument()],
    project: ProjectOption = None,
    config_path: ConfigOption = None,
    profile: ProfileOption = None,
    command: CommandOption = None,
) -> None:
    _, loaded, context = _resolve_runtime(project, config_path)
    adapter = _selected_adapters(loaded, [agent_name])[0]
    selected_profile = _profile_for(adapter, loaded, context, profile)
    inline = adapter.inline_args(selected_profile, context)
    target_command = shlex.split(command or loaded.config.resolve_command(adapter.metadata.key))
    output = shlex.join([*inline.args]) if not ctx.args else shlex.join([*target_command, *inline.args, *ctx.args])
    if not inline.args and not ctx.args:
        console.print(t("inline.no_args"))
    else:
        console.print(output)
    _print_warnings(inline.warnings)


@app.command("exec", help=t("exec.help"), context_settings={"allow_extra_args": True, "ignore_unknown_options": True})
def exec_command(
    ctx: typer.Context,
    agent_name: Annotated[str, typer.Argument()],
    project: ProjectOption = None,
    config_path: ConfigOption = None,
    profile: ProfileOption = None,
    command: CommandOption = None,
) -> None:
    _, loaded, context = _resolve_runtime(project, config_path)
    adapter = _selected_adapters(loaded, [agent_name])[0]
    selected_profile = _profile_for(adapter, loaded, context, profile)
    inline = adapter.inline_args(selected_profile, context)
    base_command = shlex.split(command or loaded.config.resolve_command(adapter.metadata.key))
    final_command = [*base_command, *inline.args, *ctx.args]
    console.print(f"{t('exec.running')}: {shlex.join(final_command)}")
    _print_warnings(inline.warnings)
    completed = subprocess.run(final_command, check=False)
    raise typer.Exit(completed.returncode)


def main() -> None:
    app()



from __future__ import annotations

import difflib
import json
from dataclasses import dataclass, field
from pathlib import Path
from typing import Any

from gperm.formats import write_data, write_text


def _normalize(value: Any) -> Any:
    if isinstance(value, dict):
        return {key: _normalize(value[key]) for key in sorted(value)}
    if isinstance(value, list):
        return [_normalize(item) for item in value]
    return value


def _pretty(value: Any) -> str:
    if isinstance(value, str):
        return value
    return json.dumps(_normalize(value), indent=2, ensure_ascii=False, sort_keys=True)


@dataclass(slots=True)
class OperationPlan:
    agent: str
    scope: str
    path: Path
    file_format: str
    label: str
    current_full: Any
    desired_full: Any
    current_managed: Any
    desired_managed: Any
    warnings: list[str] = field(default_factory=list)

    def changed(self) -> bool:
        return _normalize(self.current_managed) != _normalize(self.desired_managed)

    def apply(self) -> None:
        if self.file_format == "text":
            write_text(self.path, str(self.desired_full))
            return
        write_data(self.path, self.file_format, self.desired_full)

    def diff_text(self) -> str:
        before = _pretty(self.current_managed).splitlines()
        after = _pretty(self.desired_managed).splitlines()
        return "\n".join(
            difflib.unified_diff(
                before,
                after,
                fromfile=f"{self.label}:current",
                tofile=f"{self.label}:desired",
                lineterm="",
            )
        )



from __future__ import annotations

from copy import deepcopy
from pathlib import Path

from gperm.adapters.base import AdapterContext, AdapterMetadata, InlineResult, resolved_directories
from gperm.formats import read_data, write_text
from gperm.model import PermissionProfile
from gperm.operations import OperationPlan
from gperm.util import home_dir


def _settings_path(context: AdapterContext, *, project: bool) -> Path:
    if project:
        return context.project_root / ".gemini" / "settings.json"
    return home_dir(context.env) / ".gemini" / "settings.json"


def _trusted_path(context: AdapterContext) -> Path:
    return home_dir(context.env) / ".gemini" / "trustedFolders.json"


def _approval_mode(profile: PermissionProfile, *, inline: bool) -> tuple[str, list[str]]:
    warnings: list[str] = []
    if profile.approval == "auto-edit":
        return "auto_edit", warnings
    if profile.approval == "plan":
        return "plan", warnings
    if profile.approval == "full-auto":
        if inline:
            return "yolo", warnings
        warnings.append("Gemini persistent settings do not support yolo directly; gperm approximates it with auto_edit plus policy rules.")
        return "auto_edit", warnings
    return "default", warnings


def _sandbox_value(profile: PermissionProfile) -> bool:
    return profile.sandbox != "danger-full-access"


def _generated_policy_path(context: AdapterContext, *, project: bool, profile: PermissionProfile) -> Path:
    root = context.project_gperm_dir if project else context.user_gperm_dir
    return root / "generated" / "gemini" / f"{'project' if project else 'global'}-{profile.name}.toml"


def _policy_rules(profile: PermissionProfile) -> tuple[str, list[str]]:
    lines: list[str] = []
    warnings: list[str] = []
    priority = 300

    def emit(tool_name: str, decision: str, command_prefix: str | None = None) -> None:
        nonlocal priority
        lines.append("[[rule]]")
        lines.append(f'toolName = "{tool_name}"')
        if command_prefix is not None:
            lines.append(f'commandPrefix = "{command_prefix}"')
        lines.append(f'decision = "{decision}"')
        lines.append(f"priority = {priority}")
        lines.append("")
        priority -= 10

    for command in profile.deny_shell:
        emit("run_shell_command", "deny", command)
    for command in profile.ask_shell:
        emit("run_shell_command", "ask_user", command)
    for command in profile.allow_shell:
        emit("run_shell_command", "allow", command)

    tool_map = {
        "edit": "replace",
        "write": "write_file",
    }

    for tool in profile.deny_tools:
        mapped = tool_map.get(tool)
        if mapped:
            emit(mapped, "deny")
        elif tool not in {"shell"}:
            warnings.append(f"Gemini policy engine mapping is not implemented for generic tool '{tool}'.")
    for tool in profile.ask_tools:
        mapped = tool_map.get(tool)
        if mapped:
            emit(mapped, "ask_user")
        elif tool not in {"shell"}:
            warnings.append(f"Gemini policy engine mapping is not implemented for generic tool '{tool}'.")
    for tool in profile.allow_tools:
        mapped = tool_map.get(tool)
        if mapped:
            emit(mapped, "allow")
        elif tool not in {"shell"}:
            warnings.append(f"Gemini policy engine mapping is not implemented for generic tool '{tool}'.")

    if "webfetch" in profile.allow_tools or "webfetch" in profile.deny_tools or "webfetch" in profile.ask_tools:
        warnings.append("Gemini webfetch policy mapping is not implemented by gperm.")
    if "read" in profile.allow_tools or "read" in profile.deny_tools or "read" in profile.ask_tools:
        warnings.append("Gemini read-only tool policy mapping is not implemented by gperm.")

    return "\n".join(lines).rstrip() + ("\n" if lines else ""), warnings


class GeminiAdapter:
    metadata = AdapterMetadata(
        key="geminicli",
        display_name="Gemini CLI",
        aliases=("gemini", "geminicli"),
        default_command="gemini",
        tested_version="0.31.0",
        global_support="native",
        project_support="native",
        inline_support="native",
        docs=(
            "https://geminicli.com/docs/reference/policy-engine/",
            "https://geminicli.com/docs/get-started/configuration/",
        ),
    )

    def _settings_operation(self, path: Path, current: dict[str, object], desired: dict[str, object], scope: str) -> OperationPlan:
        managed = {
            "general": {"defaultApprovalMode": dict(current.get("general", {})).get("defaultApprovalMode")},
            "tools": {"sandbox": dict(current.get("tools", {})).get("sandbox")},
            "context": {"includeDirectories": dict(current.get("context", {})).get("includeDirectories", [])},
            "policyPaths": current.get("policyPaths", []),
        }
        merged = deepcopy(current)
        merged["general"] = dict(merged.get("general", {})) | desired["general"]
        merged["tools"] = dict(merged.get("tools", {})) | desired["tools"]
        merged["context"] = dict(merged.get("context", {})) | desired["context"]
        merged["policyPaths"] = desired["policyPaths"]
        return OperationPlan(
            agent=self.metadata.key,
            scope=scope,
            path=path,
            file_format="json",
            label=f"Gemini {scope} settings",
            current_full=current,
            desired_full=merged,
            current_managed=managed,
            desired_managed=desired,
        )

    def build_operations(
        self,
        profile: PermissionProfile,
        context: AdapterContext,
        *,
        include_global: bool,
        include_project: bool,
    ) -> tuple[list[OperationPlan], list[str]]:
        operations: list[OperationPlan] = []
        warnings: list[str] = []

        for project_flag in (False, True):
            if project_flag and not include_project:
                continue
            if not project_flag and not include_global:
                continue

            approval_mode, mode_warnings = _approval_mode(profile, inline=False)
            warnings.extend(mode_warnings)
            include_dirs = resolved_directories(profile.include_directories, context)
            policy_content, policy_warnings = _policy_rules(profile)
            warnings.extend(policy_warnings)
            policy_paths: list[str] = []
            if policy_content:
                policy_path = _generated_policy_path(context, project=project_flag, profile=profile)
                current_policy = policy_path.read_text(encoding="utf-8") if policy_path.exists() else ""
                operations.append(
                    OperationPlan(
                        agent=self.metadata.key,
                        scope="project" if project_flag else "global",
                        path=policy_path,
                        file_format="text",
                        label=f"Gemini {'project' if project_flag else 'global'} generated policy",
                        current_full=current_policy,
                        desired_full=policy_content,
                        current_managed=current_policy,
                        desired_managed=policy_content,
                    )
                )
                policy_paths = [str(policy_path)]

            desired = {
                "general": {"defaultApprovalMode": approval_mode},
                "tools": {"sandbox": _sandbox_value(profile)},
                "context": {"includeDirectories": include_dirs},
                "policyPaths": policy_paths,
            }
            path = _settings_path(context, project=project_flag)
            current = dict(read_data(path, "json"))
            operations.append(self._settings_operation(path, current, desired, "project" if project_flag else "global"))

        if include_project:
            trusted_path = _trusted_path(context)
            current = dict(read_data(trusted_path, "json"))
            desired_full = deepcopy(current)
            desired_full[str(context.project_root)] = "TRUST_FOLDER" if profile.trust else "DO_NOT_TRUST"
            operations.append(
                OperationPlan(
                    agent=self.metadata.key,
                    scope="project",
                    path=trusted_path,
                    file_format="json",
                    label="Gemini trusted folders",
                    current_full=current,
                    desired_full=desired_full,
                    current_managed={str(context.project_root): current.get(str(context.project_root))},
                    desired_managed={str(context.project_root): desired_full[str(context.project_root)]},
                )
            )

        return operations, warnings

    def inline_args(self, profile: PermissionProfile, context: AdapterContext) -> InlineResult:
        approval_mode, warnings = _approval_mode(profile, inline=True)
        args = ["--approval-mode", approval_mode, f"--sandbox={str(_sandbox_value(profile)).lower()}"]
        for directory in resolved_directories(profile.include_directories, context):
            args.extend(["--include-directories", directory])

        policy_content, policy_warnings = _policy_rules(profile)
        warnings.extend(policy_warnings)
        if policy_content:
            policy_path = _generated_policy_path(context, project=True, profile=profile)
            write_text(policy_path, policy_content)
            args.extend(["--policy", str(policy_path)])
        return InlineResult(args=args, warnings=warnings)



from __future__ import annotations

from collections.abc import Mapping

from gperm.util import detect_locale


MESSAGES: dict[str, dict[str, str]] = {
    "en": {
        "app.help": "Manage and sync permission settings across AI coding agent CLIs.",
        "version.help": "Show version and exit.",
        "project.help": "Project directory to inspect or update.",
        "agent.help": "Target agent. May be passed multiple times.",
        "level.help": "Target settings scope.",
        "format.help": "Output format.",
        "yes.help": "Apply changes without interactive confirmation.",
        "dry_run.help": "Preview changes without writing files.",
        "profile.help": "Override the configured profile for this command.",
        "command.help": "Override the configured agent command.",
        "config.help": "Use an explicit gperm config file instead of auto-discovery.",
        "config_app.help": "Inspect or initialize gperm config.",
        "config.show.help": "Show effective gperm config and active config sources.",
        "config.init.help": "Write a starter config file.",
        "import.help": "Import a native Claude or OpenCode settings file into gperm config.",
        "doctor.help": "Run setup diagnostics for gperm and supported agent CLIs.",
        "agents.help": "List supported agents and compatibility notes.",
        "check.help": "Check whether agent settings match the selected gperm profiles.",
        "sync.help": "Synchronize native agent settings to the selected gperm profiles.",
        "inline.help": "Print inline CLI flags for the selected agent profile.",
        "exec.help": "Run an agent command with inline flags generated from the selected profile.",
        "scope.global": "global",
        "scope.project": "project",
        "scope.all": "all",
        "source.user": "user config",
        "source.project": "project override",
        "source.defaults": "built-in defaults",
        "status.match": "match",
        "status.drift": "drift",
        "status.skipped": "skipped",
        "status.unsupported": "unsupported",
        "config.active_sources": "Active config sources",
        "config.project_root": "Project root",
        "config.default_profile": "Default profile",
        "config.resolved_profiles": "Resolved profiles",
        "config.ignored": "Ignored",
        "sync.no_changes": "No changes required.",
        "sync.pending_changes": "Planned changes",
        "sync.confirm": "Apply these changes?",
        "sync.applied": "Applied",
        "sync.skipped": "Skipped",
        "sync.aborted": "Aborted.",
        "inline.no_args": "This agent has no inline permission flags to expand.",
        "inline.command": "Full command",
        "inline.flags": "Flags",
        "exec.running": "Running",
        "check.summary": "Check summary",
        "warn.header": "Warnings",
        "note.header": "Notes",
        "unsupported.project": "Native project-level persistence is not available for this agent.",
        "ignored.project": "This project is ignored by the active gperm config.",
        "init.exists": "Config already exists. Use --force to overwrite it.",
        "init.wrote": "Wrote",
        "init.skipped": "Config already exists. Skipped.",
        "origin.header": "Using",
        "doctor.title": "Setup diagnostics",
        "doctor.summary": "Summary",
        "doctor.config": "Config",
        "doctor.agents": "Agents",
        "doctor.item": "Item",
        "doctor.status": "Status",
        "doctor.details": "Details",
        "doctor.ok": "ok",
        "doctor.warn": "warn",
        "doctor.fail": "fail",
        "doctor.found": "found",
        "doctor.missing": "missing",
        "doctor.profile": "profile",
        "doctor.command": "command",
        "doctor.active_sources": "active sources",
        "doctor.using_defaults": "using built-in defaults only",
        "doctor.project_ignored": "project is ignored by gperm",
        "doctor.no_issues": "No blocking issues detected.",
        "import.wrote": "Imported settings into",
        "import.profile": "Imported profile",
        "import.source": "Source file",
        "import.target": "Target config",
        "import.scope": "Target scope",
    },
    "ja": {
        "app.help": "AI coding agent CLI の権限設定を一元管理して同期します。",
        "version.help": "バージョンを表示して終了します。",
        "project.help": "確認または更新対象のプロジェクトディレクトリです。",
        "agent.help": "対象の agent です。複数指定できます。",
        "level.help": "対象の設定スコープです。",
        "format.help": "出力形式です。",
        "yes.help": "確認なしで変更を適用します。",
        "dry_run.help": "書き込まずに変更内容だけ表示します。",
        "profile.help": "このコマンドで使う profile を設定値より優先して指定します。",
        "command.help": "設定された agent command をこのコマンドだけ上書きします。",
        "config.help": "自動検出の代わりに gperm config を明示指定します。",
        "config_app.help": "gperm の設定を確認または初期化します。",
        "config.show.help": "有効な gperm 設定と利用中の設定ソースを表示します。",
        "config.init.help": "初期設定ファイルを書き出します。",
        "import.help": "Claude または OpenCode のネイティブ設定ファイルを gperm 設定へ取り込みます。",
        "doctor.help": "gperm と対応 agent CLI のセットアップ診断を行います。",
        "agents.help": "対応 agent と互換性メモを表示します。",
        "check.help": "各 agent の設定が選択された gperm profile と一致しているか確認します。",
        "sync.help": "選択された gperm profile に各 agent のネイティブ設定を同期します。",
        "inline.help": "選択した agent profile を CLI フラグへ展開して表示します。",
        "exec.help": "選択した profile から生成したフラグ付きで agent command を実行します。",
        "scope.global": "global",
        "scope.project": "project",
        "scope.all": "all",
        "source.user": "ユーザー設定",
        "source.project": "プロジェクト上書き設定",
        "source.defaults": "組み込み既定値",
        "status.match": "一致",
        "status.drift": "差分あり",
        "status.skipped": "スキップ",
        "status.unsupported": "未対応",
        "config.active_sources": "有効な設定ソース",
        "config.project_root": "プロジェクトルート",
        "config.default_profile": "既定 profile",
        "config.resolved_profiles": "解決後 profile",
        "config.ignored": "無視対象",
        "sync.no_changes": "変更はありません。",
        "sync.pending_changes": "適用予定の変更",
        "sync.confirm": "これらの変更を適用しますか？",
        "sync.applied": "適用しました",
        "sync.skipped": "スキップしました",
        "sync.aborted": "中止しました。",
        "inline.no_args": "この agent は展開できる inline 権限フラグを持っていません。",
        "inline.command": "実行コマンド",
        "inline.flags": "フラグ",
        "exec.running": "実行中",
        "check.summary": "確認結果",
        "warn.header": "警告",
        "note.header": "補足",
        "unsupported.project": "この agent にはネイティブな project-level 永続設定がありません。",
        "ignored.project": "このプロジェクトは有効な gperm 設定で無視対象です。",
        "init.exists": "設定ファイルは既に存在します。上書きするには --force を使ってください。",
        "init.wrote": "書き込みました",
        "init.skipped": "設定ファイルは既に存在するため、スキップしました。",
        "origin.header": "使用中",
        "doctor.title": "セットアップ診断",
        "doctor.summary": "概要",
        "doctor.config": "設定",
        "doctor.agents": "Agent",
        "doctor.item": "項目",
        "doctor.status": "状態",
        "doctor.details": "詳細",
        "doctor.ok": "正常",
        "doctor.warn": "警告",
        "doctor.fail": "異常",
        "doctor.found": "検出",
        "doctor.missing": "未検出",
        "doctor.profile": "profile",
        "doctor.command": "command",
        "doctor.active_sources": "有効ソース",
        "doctor.using_defaults": "組み込みデフォルトのみを使用中",
        "doctor.project_ignored": "この project は gperm の無視対象です",
        "doctor.no_issues": "致命的な問題は見つかりませんでした。",
        "import.wrote": "設定を取り込みました",
        "import.profile": "取り込み profile",
        "import.source": "入力ファイル",
        "import.target": "出力 config",
        "import.scope": "出力スコープ",
    },
}


class Translator:
    def __init__(self, locale: str) -> None:
        self.locale = locale if locale in MESSAGES else "en"

    @classmethod
    def from_env(cls, env: Mapping[str, str] | None = None) -> "Translator":
        return cls(detect_locale(env))

    def text(self, key: str) -> str:
        return MESSAGES.get(self.locale, MESSAGES["en"]).get(key, MESSAGES["en"].get(key, key))



from __future__ import annotations

import tomllib
from dataclasses import dataclass
from pathlib import Path
from typing import Any

from gperm.model import AgentConfig, ConfigSource, GPermConfig, KNOWN_AGENTS, PermissionProfile, ProjectRule, ProjectSettings
from gperm.sample import DEFAULT_CONFIG_TOML
from gperm.util import deep_merge, normalize_agent_name, xdg_config_home


DEFAULT_CONFIG_DATA: dict[str, object] = tomllib.loads(DEFAULT_CONFIG_TOML)


@dataclass(slots=True)
class ConfigLoadResult:
    config: GPermConfig
    project_root: Path
    user_config_path: Path
    project_config_path: Path
    user_config_dir: Path
    project_config_dir: Path


def default_user_config_path(env: dict[str, str]) -> Path:
    return xdg_config_home(env) / "gperm" / "config.toml"


def default_project_config_path(project_root: Path) -> Path:
    return project_root / ".gperm" / "config.toml"


def _load_toml(path: Path) -> dict[str, object]:
    return tomllib.loads(path.read_text(encoding="utf-8"))


def _build_config(data: dict[str, Any], sources: list[ConfigSource]) -> GPermConfig:
    profiles_source = dict(data.get("profiles", {}))
    profiles = {
        str(name): PermissionProfile.from_dict(str(name), dict(value))
        for name, value in profiles_source.items()
    }

    if not profiles:
        profiles = {
            str(name): PermissionProfile.from_dict(str(name), dict(value))
            for name, value in dict(DEFAULT_CONFIG_DATA["profiles"]).items()
        }

    agents_source = dict(data.get("agents", {}))
    agents = {
        normalize_agent_name(str(name)): AgentConfig.from_dict(str(name), dict(value))
        for name, value in agents_source.items()
    }
    for name in KNOWN_AGENTS:
        agents.setdefault(name, AgentConfig(name=name))

    project_rules = [ProjectRule.from_dict(dict(item)) for item in list(data.get("project_rules", []))]
    project = ProjectSettings.from_dict(dict(data.get("project", {})))

    return GPermConfig(
        version=int(data.get("version", 1)),
        default_profile=str(data.get("default_profile", "balanced")),
        ignored_projects=[str(item) for item in data.get("ignored_projects", [])],
        profiles=profiles,
        agents=agents,
        project_rules=project_rules,
        project=project,
        sources=sources,
    )


def load_config(
    *,
    project_root: Path,
    env: dict[str, str],
    explicit_config: Path | None = None,
) -> ConfigLoadResult:
    user_path = default_user_config_path(env)
    project_path = default_project_config_path(project_root)

    if explicit_config is not None:
        merged = deep_merge(DEFAULT_CONFIG_DATA, _load_toml(explicit_config))
        config = _build_config(merged, [ConfigSource(label="explicit config", path=explicit_config)])
        return ConfigLoadResult(
            config=config,
            project_root=project_root,
            user_config_path=explicit_config,
            project_config_path=project_path,
            user_config_dir=explicit_config.parent,
            project_config_dir=project_path.parent,
        )

    merged: object = DEFAULT_CONFIG_DATA
    sources: list[ConfigSource] = [ConfigSource(label="built-in defaults", path=None)]

    if user_path.exists():
        merged = deep_merge(merged, _load_toml(user_path))
        sources = [ConfigSource(label="user config", path=user_path)]

    if project_path.exists():
        merged = deep_merge(merged, _load_toml(project_path))
        sources.append(ConfigSource(label="project override", path=project_path))

    config = _build_config(dict(merged), sources)
    return ConfigLoadResult(
        config=config,
        project_root=project_root,
        user_config_path=user_path,
        project_config_path=project_path,
        user_config_dir=user_path.parent,
        project_config_dir=project_path.parent,
    )


def render_config_for_display(config: GPermConfig, project_root: Path, env: dict[str, str]) -> dict[str, object]:
    resolved_profiles = {
        agent: config.resolve_profile_name(agent, project_root, env)
        for agent in config.enabled_agents()
    }
    payload = config.to_dict()
    payload["resolved_profiles"] = resolved_profiles
    payload["ignored"] = config.is_ignored(project_root, env)
    payload["project_root"] = str(project_root)
    payload["sources"] = [
        {"label": source.label, "path": str(source.path) if source.path else None}
        for source in config.sources
    ]
    return payload


def write_sample_config(path: Path, *, force: bool = False) -> None:
    if path.exists() and not force:
        raise FileExistsError(str(path))
    path.parent.mkdir(parents=True, exist_ok=True)
    path.write_text(DEFAULT_CONFIG_TOML, encoding="utf-8")



from __future__ import annotations

import os
from collections.abc import Iterable, Mapping
from copy import deepcopy
from fnmatch import fnmatch
from pathlib import Path


AGENT_ALIASES = {
    "claude": "claude",
    "codex": "codex",
    "gemini": "geminicli",
    "geminicli": "geminicli",
    "copilot": "copilot",
    "copilotcli": "copilot",
    "copilot-cli": "copilot",
    "opencode": "opencode",
    "antigravity": "antigravity",
}


def normalize_agent_name(value: str) -> str:
    key = value.strip().lower().replace(" ", "").replace("_", "").replace("-", "")
    return AGENT_ALIASES.get(key, value.strip().lower())


def detect_locale(env: Mapping[str, str] | None = None) -> str:
    source = env or os.environ
    for name in ("GPERM_LANG", "LC_ALL", "LC_MESSAGES", "LANG"):
        value = source.get(name, "").strip().lower()
        if value.startswith("ja"):
            return "ja"
        if value:
            return "en"
    return "en"


def home_dir(env: Mapping[str, str] | None = None) -> Path:
    source = env or os.environ
    return Path(source.get("HOME", str(Path.home()))).expanduser()


def xdg_config_home(env: Mapping[str, str] | None = None) -> Path:
    source = env or os.environ
    configured = source.get("XDG_CONFIG_HOME")
    if configured:
        return Path(configured).expanduser()
    return home_dir(source) / ".config"


def expand_path(raw: str, *, env: Mapping[str, str] | None = None, base: Path | None = None) -> Path:
    source = env or os.environ
    expanded = os.path.expandvars(raw)
    path = Path(expanded).expanduser()
    if not path.is_absolute() and base is not None:
        path = base / path
    return path.resolve()


def deep_merge(base: object, overlay: object) -> object:
    if isinstance(base, Mapping) and isinstance(overlay, Mapping):
        merged: dict[str, object] = {str(key): deepcopy(value) for key, value in base.items()}
        for key, value in overlay.items():
            text_key = str(key)
            if text_key in merged:
                merged[text_key] = deep_merge(merged[text_key], value)
            else:
                merged[text_key] = deepcopy(value)
        return merged
    return deepcopy(overlay)


def uniq(items: Iterable[str]) -> list[str]:
    result: list[str] = []
    seen: set[str] = set()
    for item in items:
        if item not in seen:
            seen.add(item)
            result.append(item)
    return result


def path_matches(selector: str, project_root: Path, env: Mapping[str, str] | None = None) -> bool:
    if any(token in selector for token in ("*", "?", "[")):
        expanded = os.path.expandvars(selector)
        pattern = str(Path(expanded).expanduser())
        return fnmatch(str(project_root), pattern)

    expected = expand_path(selector, env=env)
    return project_root == expected or project_root.is_relative_to(expected)


def shell_command_variants(command: str) -> list[str]:
    normalized = command.strip()
    if not normalized:
        return []
    if any(token in normalized for token in ("*", "?", "[")):
        return [normalized]
    return uniq([normalized, f"{normalized}:*"])



from __future__ import annotations

from pathlib import Path

from gperm.adapters import get_adapter
from gperm.adapters.base import AdapterContext
from gperm.model import PermissionProfile


def make_context(tmp_path: Path) -> AdapterContext:
    home = tmp_path / "home"
    project = tmp_path / "project"
    home.mkdir()
    project.mkdir()
    env = {"HOME": str(home), "XDG_CONFIG_HOME": str(home / ".config")}
    return AdapterContext(
        env=env,
        project_root=project,
        user_gperm_dir=Path(env["XDG_CONFIG_HOME"]) / "gperm",
        project_gperm_dir=project / ".gperm",
    )


def test_claude_sync_preserves_unrelated_settings(tmp_path: Path) -> None:
    context = make_context(tmp_path)
    settings_path = Path(context.env["HOME"]) / ".claude" / "settings.json"
    settings_path.parent.mkdir(parents=True)
    settings_path.write_text('{"AWS_REGION":"ap-northeast-1"}', encoding="utf-8")

    adapter = get_adapter("claude")
    profile = PermissionProfile(name="balanced", allow_shell=["git status"], deny_shell=["git push"])
    operations, warnings = adapter.build_operations(profile, context, include_global=True, include_project=False)

    assert not warnings
    desired = operations[0].desired_full
    assert desired["AWS_REGION"] == "ap-northeast-1"
    assert "permissions" in desired
    assert "Bash(git status)" in desired["permissions"]["allow"]
    assert "Bash(git push)" in desired["permissions"]["deny"]


def test_gemini_builds_generated_policy_and_trust_entry(tmp_path: Path) -> None:
    context = make_context(tmp_path)
    adapter = get_adapter("gemini")
    profile = PermissionProfile(name="balanced", trust=True, allow_shell=["git status"], deny_shell=["git push"])
    operations, warnings = adapter.build_operations(profile, context, include_global=False, include_project=True)

    assert any("Gemini project settings" == operation.label for operation in operations)
    assert any("generated policy" in operation.label for operation in operations)
    trusted = next(operation for operation in operations if operation.label == "Gemini trusted folders")
    assert trusted.desired_managed[str(context.project_root)] == "TRUST_FOLDER"
    assert not any("not implemented" in warning for warning in warnings)


def test_copilot_inline_patterns_include_add_dir(tmp_path: Path) -> None:
    context = make_context(tmp_path)
    adapter = get_adapter("copilot")
    profile = PermissionProfile(name="balanced", trust=True, allow_shell=["git status"], deny_shell=["git push"])
    inline = adapter.inline_args(profile, context)

    assert "--allow-tool" in inline.args
    assert "shell(git status)" in inline.args
    assert "shell(git push)" in inline.args
    assert "--add-dir" in inline.args
    assert str(context.project_root) in inline.args


def test_codex_project_sync_preserves_other_projects(tmp_path: Path) -> None:
    context = make_context(tmp_path)
    config_path = Path(context.env["HOME"]) / ".codex" / "config.toml"
    config_path.parent.mkdir(parents=True)
    config_path.write_text(
        'model = "gpt-5"\n'
        '[projects."/tmp/other"]\n'
        'trust_level = "trusted"\n',
        encoding="utf-8",
    )

    adapter = get_adapter("codex")
    profile = PermissionProfile(name="balanced", trust=True, sandbox="workspace-write", approval="default")
    operations, _ = adapter.build_operations(profile, context, include_global=False, include_project=True)
    global_op = next(operation for operation in operations if operation.label == "Codex global config")

    assert global_op.desired_full["model"] == "gpt-5"
    assert global_op.desired_full["projects"]["/tmp/other"]["trust_level"] == "trusted"
    assert global_op.desired_full["projects"][str(context.project_root)]["trust_level"] == "trusted"


def test_opencode_preserves_unrelated_settings(tmp_path: Path) -> None:
    context = make_context(tmp_path)
    opencode_root = Path(context.env["XDG_CONFIG_HOME"]) / "opencode"
    config_path = opencode_root / "opencode.json"
    config_path.parent.mkdir(parents=True)
    config_path.write_text('{"share":"disabled"}', encoding="utf-8")

    adapter = get_adapter("opencode")
    profile = PermissionProfile(name="safe", approval="plan", sandbox="read-only")
    operations, _ = adapter.build_operations(profile, context, include_global=True, include_project=False)
    desired = operations[0].desired_full

    assert desired["share"] == "disabled"
    assert desired["permission"]["edit"] == "deny"



from __future__ import annotations

from copy import deepcopy
from pathlib import Path

from gperm.adapters.base import AdapterContext, AdapterMetadata, InlineResult, resolved_directories
from gperm.formats import read_data
from gperm.model import PermissionProfile
from gperm.operations import OperationPlan
from gperm.util import home_dir


def _approval_policy(profile: PermissionProfile) -> tuple[str, list[str]]:
    warnings: list[str] = []
    if profile.approval == "full-auto":
        return "never", warnings
    if profile.approval == "plan":
        return "untrusted", warnings
    if profile.approval == "auto-edit":
        warnings.append("Codex has no auto-edit-only approval policy; gperm maps it to on-request.")
    return "on-request", warnings


class CodexAdapter:
    metadata = AdapterMetadata(
        key="codex",
        display_name="Codex CLI",
        aliases=("codex",),
        default_command="codex",
        tested_version="0.112.0",
        global_support="native",
        project_support="native",
        inline_support="native",
        docs=(
            "https://developers.openai.com/codex/config-basics",
            "https://developers.openai.com/codex/config-reference",
            "https://developers.openai.com/codex/sandbox-and-approvals",
        ),
    )

    def _global_path(self, context: AdapterContext) -> Path:
        return home_dir(context.env) / ".codex" / "config.toml"

    def _project_path(self, context: AdapterContext) -> Path:
        return context.project_root / ".codex" / "config.toml"

    def build_operations(
        self,
        profile: PermissionProfile,
        context: AdapterContext,
        *,
        include_global: bool,
        include_project: bool,
    ) -> tuple[list[OperationPlan], list[str]]:
        operations: list[OperationPlan] = []
        warnings: list[str] = []
        warnings.extend(
            warning
            for warning in (
                "Codex does not have native URL allow/deny persistence." if profile.allow_urls or profile.deny_urls else "",
                "Codex does not have native shell/tool allow/deny persistence; gperm sync manages approval and sandbox instead."
                if profile.allow_tools or profile.deny_tools or profile.ask_tools or profile.allow_shell or profile.deny_shell or profile.ask_shell
                else "",
                "Codex does not have persistent include-directory settings; use gperm inline/exec for --add-dir."
                if profile.include_directories
                else "",
            )
            if warning
        )

        approval_policy, policy_warnings = _approval_policy(profile)
        warnings.extend(policy_warnings)

        if include_global or include_project:
            path = self._global_path(context)
            current = dict(read_data(path, "toml"))
            desired_full = deepcopy(current)
            current_managed: dict[str, object] = {}
            desired_managed: dict[str, object] = {}

            if include_global:
                desired_full["approval_policy"] = approval_policy
                desired_full["sandbox_mode"] = profile.sandbox
                current_managed["approval_policy"] = current.get("approval_policy")
                current_managed["sandbox_mode"] = current.get("sandbox_mode")
                desired_managed["approval_policy"] = approval_policy
                desired_managed["sandbox_mode"] = profile.sandbox

            if include_project:
                projects = dict(desired_full.get("projects", {}))
                project_entry = dict(projects.get(str(context.project_root), {}))
                current_projects = dict(current.get("projects", {}))
                current_project_entry = dict(current_projects.get(str(context.project_root), {}))
                current_managed["project_trust"] = current_project_entry.get("trust_level")
                if profile.trust:
                    project_entry["trust_level"] = "trusted"
                    projects[str(context.project_root)] = project_entry
                    desired_managed["project_trust"] = "trusted"
                else:
                    projects.pop(str(context.project_root), None)
                    desired_managed["project_trust"] = None
                desired_full["projects"] = projects

            operations.append(
                OperationPlan(
                    agent=self.metadata.key,
                    scope="global/project" if include_global and include_project else ("project" if include_project else "global"),
                    path=path,
                    file_format="toml",
                    label="Codex global config",
                    current_full=current,
                    desired_full=desired_full,
                    current_managed=current_managed,
                    desired_managed=desired_managed,
                )
            )

        if include_project:
            path = self._project_path(context)
            current = dict(read_data(path, "toml"))
            desired_full = deepcopy(current)
            desired_full["approval_policy"] = approval_policy
            desired_full["sandbox_mode"] = profile.sandbox
            operations.append(
                OperationPlan(
                    agent=self.metadata.key,
                    scope="project",
                    path=path,
                    file_format="toml",
                    label="Codex project config",
                    current_full=current,
                    desired_full=desired_full,
                    current_managed={
                        "approval_policy": current.get("approval_policy"),
                        "sandbox_mode": current.get("sandbox_mode"),
                    },
                    desired_managed={
                        "approval_policy": approval_policy,
                        "sandbox_mode": profile.sandbox,
                    },
                )
            )

        return operations, warnings

    def inline_args(self, profile: PermissionProfile, context: AdapterContext) -> InlineResult:
        approval_policy, warnings = _approval_policy(profile)
        args = ["-a", approval_policy, "-s", profile.sandbox]
        for directory in resolved_directories(profile.include_directories, context):
            args.extend(["--add-dir", directory])
        return InlineResult(args=args, warnings=warnings)



from __future__ import annotations

from dataclasses import dataclass, field
from pathlib import Path

from gperm.util import normalize_agent_name, path_matches


KNOWN_AGENTS = (
    "claude",
    "geminicli",
    "copilot",
    "codex",
    "opencode",
    "antigravity",
)


@dataclass(slots=True)
class PermissionProfile:
    name: str
    approval: str = "default"
    sandbox: str = "workspace-write"
    trust: bool = False
    include_directories: list[str] = field(default_factory=list)
    allow_tools: list[str] = field(default_factory=list)
    deny_tools: list[str] = field(default_factory=list)
    ask_tools: list[str] = field(default_factory=list)
    allow_shell: list[str] = field(default_factory=list)
    deny_shell: list[str] = field(default_factory=list)
    ask_shell: list[str] = field(default_factory=list)
    allow_urls: list[str] = field(default_factory=list)
    deny_urls: list[str] = field(default_factory=list)
    notes: str = ""

    @classmethod
    def from_dict(cls, name: str, data: dict[str, object]) -> "PermissionProfile":
        return cls(
            name=name,
            approval=str(data.get("approval", "default")),
            sandbox=str(data.get("sandbox", "workspace-write")),
            trust=bool(data.get("trust", False)),
            include_directories=[str(item) for item in data.get("include_directories", [])],
            allow_tools=[str(item) for item in data.get("allow_tools", [])],
            deny_tools=[str(item) for item in data.get("deny_tools", [])],
            ask_tools=[str(item) for item in data.get("ask_tools", [])],
            allow_shell=[str(item) for item in data.get("allow_shell", [])],
            deny_shell=[str(item) for item in data.get("deny_shell", [])],
            ask_shell=[str(item) for item in data.get("ask_shell", [])],
            allow_urls=[str(item) for item in data.get("allow_urls", [])],
            deny_urls=[str(item) for item in data.get("deny_urls", [])],
            notes=str(data.get("notes", "")),
        )

    def to_dict(self) -> dict[str, object]:
        return {
            "approval": self.approval,
            "sandbox": self.sandbox,
            "trust": self.trust,
            "include_directories": self.include_directories,
            "allow_tools": self.allow_tools,
            "deny_tools": self.deny_tools,
            "ask_tools": self.ask_tools,
            "allow_shell": self.allow_shell,
            "deny_shell": self.deny_shell,
            "ask_shell": self.ask_shell,
            "allow_urls": self.allow_urls,
            "deny_urls": self.deny_urls,
            "notes": self.notes,
        }


@dataclass(slots=True)
class AgentConfig:
    name: str
    enabled: bool = True
    profile: str | None = None
    command: str | None = None

    @classmethod
    def from_dict(cls, name: str, data: dict[str, object]) -> "AgentConfig":
        return cls(
            name=normalize_agent_name(name),
            enabled=bool(data.get("enabled", True)),
            profile=str(data["profile"]) if data.get("profile") else None,
            command=str(data["command"]) if data.get("command") else None,
        )

    def to_dict(self) -> dict[str, object]:
        result: dict[str, object] = {"enabled": self.enabled}
        if self.profile is not None:
            result["profile"] = self.profile
        if self.command is not None:
            result["command"] = self.command
        return result


@dataclass(slots=True)
class ProjectRule:
    path: str
    ignore: bool = False
    profile: str | None = None
    agents: dict[str, AgentConfig] = field(default_factory=dict)

    @classmethod
    def from_dict(cls, data: dict[str, object]) -> "ProjectRule":
        agents = {
            normalize_agent_name(name): AgentConfig.from_dict(name, value)
            for name, value in dict(data.get("agents", {})).items()
        }
        return cls(
            path=str(data["path"]),
            ignore=bool(data.get("ignore", False)),
            profile=str(data["profile"]) if data.get("profile") else None,
            agents=agents,
        )

    def matches(self, project_root: Path, env: dict[str, str]) -> bool:
        return path_matches(self.path, project_root, env)

    def to_dict(self) -> dict[str, object]:
        result: dict[str, object] = {"path": self.path}
        if self.ignore:
            result["ignore"] = True
        if self.profile is not None:
            result["profile"] = self.profile
        if self.agents:
            result["agents"] = {name: value.to_dict() for name, value in self.agents.items()}
        return result


@dataclass(slots=True)
class ProjectSettings:
    ignore: bool = False
    profile: str | None = None
    agents: dict[str, AgentConfig] = field(default_factory=dict)

    @classmethod
    def from_dict(cls, data: dict[str, object]) -> "ProjectSettings":
        agents = {
            normalize_agent_name(name): AgentConfig.from_dict(name, value)
            for name, value in dict(data.get("agents", {})).items()
        }
        return cls(
            ignore=bool(data.get("ignore", False)),
            profile=str(data["profile"]) if data.get("profile") else None,
            agents=agents,
        )

    def to_dict(self) -> dict[str, object]:
        result: dict[str, object] = {}
        if self.ignore:
            result["ignore"] = True
        if self.profile is not None:
            result["profile"] = self.profile
        if self.agents:
            result["agents"] = {name: value.to_dict() for name, value in self.agents.items()}
        return result


@dataclass(slots=True)
class ConfigSource:
    label: str
    path: Path | None


@dataclass(slots=True)
class GPermConfig:
    version: int
    default_profile: str
    ignored_projects: list[str]
    profiles: dict[str, PermissionProfile]
    agents: dict[str, AgentConfig]
    project_rules: list[ProjectRule]
    project: ProjectSettings
    sources: list[ConfigSource] = field(default_factory=list)

    def to_dict(self) -> dict[str, object]:
        return {
            "version": self.version,
            "default_profile": self.default_profile,
            "ignored_projects": self.ignored_projects,
            "profiles": {name: profile.to_dict() for name, profile in self.profiles.items()},
            "agents": {name: agent.to_dict() for name, agent in self.agents.items()},
            "project_rules": [rule.to_dict() for rule in self.project_rules],
            "project": self.project.to_dict(),
        }

    def enabled_agents(self, requested: list[str] | None = None) -> list[str]:
        candidates = requested or list(KNOWN_AGENTS)
        result: list[str] = []
        for item in candidates:
            name = normalize_agent_name(item)
            config = self.agents.get(name)
            if config and not config.enabled:
                continue
            if name in KNOWN_AGENTS and name not in result:
                result.append(name)
        return result

    def is_ignored(self, project_root: Path, env: dict[str, str]) -> bool:
        if self.project.ignore:
            return True
        for selector in self.ignored_projects:
            if path_matches(selector, project_root, env):
                return True
        for rule in self.project_rules:
            if rule.ignore and rule.matches(project_root, env):
                return True
        return False

    def resolve_profile_name(self, agent_name: str, project_root: Path, env: dict[str, str]) -> str:
        name = normalize_agent_name(agent_name)
        if name in self.project.agents and self.project.agents[name].profile:
            return self.project.agents[name].profile or self.default_profile

        for rule in reversed(self.project_rules):
            if rule.matches(project_root, env) and name in rule.agents and rule.agents[name].profile:
                return rule.agents[name].profile or self.default_profile

        if self.project.profile:
            return self.project.profile

        for rule in reversed(self.project_rules):
            if rule.matches(project_root, env) and rule.profile:
                return rule.profile

        if name in self.agents and self.agents[name].profile:
            return self.agents[name].profile or self.default_profile

        return self.default_profile

    def resolve_profile(self, agent_name: str, project_root: Path, env: dict[str, str]) -> PermissionProfile:
        profile_name = self.resolve_profile_name(agent_name, project_root, env)
        if profile_name not in self.profiles:
            raise KeyError(f"Unknown profile: {profile_name}")
        return self.profiles[profile_name]

    def resolve_command(self, agent_name: str) -> str:
        name = normalize_agent_name(agent_name)
        configured = self.agents.get(name)
        if configured and configured.command:
            return configured.command
        defaults = {
            "claude": "claude",
            "geminicli": "gemini",
            "copilot": "copilot",
            "codex": "codex",
            "opencode": "opencode",
            "antigravity": "antigravity",
        }
        return defaults[name]



from __future__ import annotations

from copy import deepcopy
from pathlib import Path

from gperm.adapters.base import AdapterContext, AdapterMetadata, InlineResult, default_decision, opencode_bash_rules
from gperm.formats import read_data
from gperm.model import PermissionProfile
from gperm.operations import OperationPlan
from gperm.util import xdg_config_home


class OpenCodeAdapter:
    metadata = AdapterMetadata(
        key="opencode",
        display_name="OpenCode",
        aliases=("opencode",),
        default_command="opencode",
        tested_version="1.2.24",
        global_support="native",
        project_support="native",
        inline_support="none",
        docs=(
            "https://opencode.ai/docs/ja/permissions/",
            "https://opencode.ai/docs/ja/configuration/",
        ),
    )

    def _path(self, context: AdapterContext, *, project: bool) -> tuple[Path, str]:
        if project:
            return context.project_root / "opencode.json", "json"
        root = xdg_config_home(context.env) / "opencode"
        jsonc_path = root / "opencode.jsonc"
        if jsonc_path.exists():
            return jsonc_path, "jsonc"
        return root / "opencode.json", "json"

    def _managed(self, profile: PermissionProfile) -> tuple[dict[str, object], list[str]]:
        warnings: list[str] = []
        permission = {
            "edit": default_decision(profile, "edit"),
            "bash": opencode_bash_rules(profile),
            "webfetch": default_decision(profile, "webfetch"),
            "external_directory": "allow" if profile.trust or profile.include_directories else default_decision(profile, "shell"),
        }
        if profile.include_directories:
            warnings.append("OpenCode does not support exact include-directory lists; gperm maps them to external_directory permission.")
        return {"permission": permission}, warnings

    def build_operations(
        self,
        profile: PermissionProfile,
        context: AdapterContext,
        *,
        include_global: bool,
        include_project: bool,
    ) -> tuple[list[OperationPlan], list[str]]:
        operations: list[OperationPlan] = []
        warnings: list[str] = []
        managed, managed_warnings = self._managed(profile)
        warnings.extend(managed_warnings)

        for project_flag in (False, True):
            if project_flag and not include_project:
                continue
            if not project_flag and not include_global:
                continue
            path, file_format = self._path(context, project=project_flag)
            current = dict(read_data(path, file_format))
            desired_full = deepcopy(current)
            desired_full["permission"] = managed["permission"]
            operations.append(
                OperationPlan(
                    agent=self.metadata.key,
                    scope="project" if project_flag else "global",
                    path=path,
                    file_format=file_format,
                    label=f"OpenCode {'project' if project_flag else 'global'} config",
                    current_full=current,
                    desired_full=desired_full,
                    current_managed={"permission": current.get("permission", {})},
                    desired_managed=managed,
                )
            )

        return operations, warnings

    def inline_args(self, profile: PermissionProfile, context: AdapterContext) -> InlineResult:
        return InlineResult(args=[], warnings=["OpenCode does not expose documented inline permission flags in gperm 0.0.1."])



from __future__ import annotations

import json
from pathlib import Path

from typer.testing import CliRunner

from gperm.cli import app


runner = CliRunner()


def test_config_show_json_reports_project_override(tmp_path: Path) -> None:
    home = tmp_path / "home"
    project = tmp_path / "project"
    user_config = home / ".config" / "gperm" / "config.toml"
    project_config = project / ".gperm" / "config.toml"
    user_config.parent.mkdir(parents=True)
    project_config.parent.mkdir(parents=True)
    project.mkdir(exist_ok=True)

    user_config.write_text(
        'version = 1\n'
        'default_profile = "balanced"\n'
        '[profiles.safe]\n'
        'approval = "plan"\n'
        'sandbox = "read-only"\n'
        '[profiles.balanced]\n'
        'approval = "default"\n'
        'sandbox = "workspace-write"\n',
        encoding="utf-8",
    )
    project_config.write_text('[project]\nprofile = "safe"\n', encoding="utf-8")

    env = {"HOME": str(home), "XDG_CONFIG_HOME": str(home / ".config")}
    result = runner.invoke(app, ["config", "show", "--project", str(project), "--format", "json"], env=env)

    assert result.exit_code == 0
    payload = json.loads(result.stdout)
    assert payload["resolved_profiles"]["claude"] == "safe"
    assert payload["sources"][0]["label"] == "user config"
    assert payload["sources"][1]["label"] == "project override"


def test_sync_dry_run_does_not_write_files(tmp_path: Path) -> None:
    home = tmp_path / "home"
    project = tmp_path / "project"
    project.mkdir()
    env = {"HOME": str(home), "XDG_CONFIG_HOME": str(home / ".config")}

    result = runner.invoke(
        app,
        ["sync", "--project", str(project), "--agent", "claude", "--level", "global", "--dry-run", "--yes"],
        env=env,
    )

    assert result.exit_code == 0
    assert not (home / ".claude" / "settings.json").exists()


def test_inline_outputs_full_command_when_extra_args_are_present(tmp_path: Path) -> None:
    home = tmp_path / "home"
    project = tmp_path / "project"
    project.mkdir()
    env = {"HOME": str(home), "XDG_CONFIG_HOME": str(home / ".config")}

    result = runner.invoke(
        app,
        ["inline", "codex", "--project", str(project), "--command", "codex", "--", "exec", "fix bug"],
        env=env,
    )

    assert result.exit_code == 0
    assert "codex -a" in result.stdout
    assert "exec" in result.stdout


def test_config_init_if_missing_is_idempotent(tmp_path: Path) -> None:
    home = tmp_path / "home"
    project = tmp_path / "project"
    project.mkdir()
    env = {"HOME": str(home), "XDG_CONFIG_HOME": str(home / ".config")}

    first = runner.invoke(app, ["config", "init"], env=env)
    second = runner.invoke(app, ["config", "init", "--if-missing"], env=env)

    assert first.exit_code == 0
    assert second.exit_code == 0
    assert "Skipped" in second.stdout or "スキップ" in second.stdout


def test_doctor_reports_missing_agent_binary(tmp_path: Path) -> None:
    home = tmp_path / "home"
    project = tmp_path / "project"
    user_config = home / ".config" / "gperm" / "config.toml"
    project.mkdir()
    user_config.parent.mkdir(parents=True)
    user_config.write_text(
        'version = 1\n'
        'default_profile = "balanced"\n'
        '[profiles.balanced]\n'
        'approval = "default"\n'
        'sandbox = "workspace-write"\n'
        '[agents.codex]\n'
        'command = "definitely-missing-codex"\n',
        encoding="utf-8",
    )

    env = {"HOME": str(home), "XDG_CONFIG_HOME": str(home / ".config"), "PATH": "/usr/bin:/bin"}
    result = runner.invoke(app, ["doctor", "--project", str(project), "--agent", "codex", "--format", "json"], env=env)

    assert result.exit_code == 1
    assert "binary not found on PATH" in result.stdout


def test_import_claude_creates_project_local_gperm_config(tmp_path: Path) -> None:
    project = tmp_path / "project"
    claude_dir = project / ".claude"
    claude_dir.mkdir(parents=True)
    source = claude_dir / "settings.json"
    source.write_text(
        json.dumps(
            {
                "defaultMode": "acceptEdits",
                "additionalDirectories": ["/tmp/shared"],
                "permissions": {
                    "allow": ["Edit", "Bash(git status)"],
                    "deny": ["Bash(git push)"],
                },
            }
        ),
        encoding="utf-8",
    )

    result = runner.invoke(app, ["import", "claude", str(source)])

    target = project / ".gperm" / "config.toml"
    content = target.read_text(encoding="utf-8")
    assert result.exit_code == 0
    assert 'imported-claude' in content
    assert 'profile = "imported-claude"' in content
    assert 'approval = "auto-edit"' in content
    assert 'allow_shell = ["git status"]' in content


def test_import_opencode_global_targets_xdg_config(tmp_path: Path) -> None:
    home = tmp_path / "home"
    config_home = home / ".config"
    opencode_root = config_home / "opencode"
    opencode_root.mkdir(parents=True)
    source = opencode_root / "opencode.jsonc"
    source.write_text(
        json.dumps(
            {
                "permission": {
                    "edit": "allow",
                    "bash": {"*": "ask", "git status*": "allow", "git push*": "deny"},
                    "webfetch": "deny",
                    "external_directory": "allow",
                }
            }
        ),
        encoding="utf-8",
    )

    env = {"HOME": str(home), "XDG_CONFIG_HOME": str(config_home)}
    result = runner.invoke(app, ["import", "opencode", str(source)], env=env)

    target = config_home / "gperm" / "config.toml"
    content = target.read_text(encoding="utf-8")
    assert result.exit_code == 0
    assert 'imported-opencode' in content
    assert '[agents.opencode]' in content
    assert 'profile = "imported-opencode"' in content
    assert 'trust = true' in content



from importlib.metadata import PackageNotFoundError, version
from pathlib import Path
import tomllib

__all__ = ["__version__"]


def _fallback_version() -> str:
    pyproject_path = Path(__file__).resolve().parents[2] / "pyproject.toml"
    data = tomllib.loads(pyproject_path.read_text(encoding="utf-8"))
    return str(data["project"]["version"])


try:
    __version__ = version("global-permission")
except PackageNotFoundError:
    __version__ = _fallback_version()



DEFAULT_CONFIG_TOML = """version = 1
default_profile = "balanced"
ignored_projects = []

[profiles.safe]
approval = "plan"
sandbox = "read-only"
trust = false
deny_tools = ["edit", "write", "shell", "webfetch"]

[profiles.balanced]
approval = "default"
sandbox = "workspace-write"
trust = true
allow_shell = ["git status", "git diff", "git show", "ls", "pwd", "cat", "rg"]
deny_shell = ["git push", "rm -rf"]

[profiles.full-auto]
approval = "full-auto"
sandbox = "danger-full-access"
trust = true

[agents.claude]
enabled = true
profile = "balanced"
command = "claude"

[agents.geminicli]
enabled = true
profile = "balanced"
command = "gemini"

[agents.copilot]
enabled = true
profile = "balanced"
command = "copilot"

[agents.codex]
enabled = true
profile = "balanced"
command = "codex"

[agents.opencode]
enabled = true
profile = "balanced"
command = "opencode"

[agents.antigravity]
enabled = true
profile = "balanced"
command = "antigravity"

[[project_rules]]
path = "~/work/secrets"
ignore = true

[[project_rules]]
path = "~/work/release"
profile = "safe"
[project_rules.agents.codex]
profile = "full-auto"
"""



from __future__ import annotations

import copy
from dataclasses import dataclass
from pathlib import Path

from gperm.config import DEFAULT_CONFIG_DATA
from gperm.formats import read_data
from gperm.model import PermissionProfile
from gperm.util import home_dir, xdg_config_home


@dataclass(slots=True)
class ImportPlan:
    agent: str
    source_path: Path
    target_path: Path
    profile_name: str
    profile: PermissionProfile
    scope: str
    warnings: list[str]


def _approval_from_claude(mode: str) -> str:
    return {
        "acceptEdits": "auto-edit",
        "bypassPermissions": "full-auto",
        "dontAsk": "full-auto",
        "plan": "plan",
        "default": "default",
    }.get(mode, "default")


def _profile_from_claude(path: Path) -> tuple[PermissionProfile, list[str]]:
    data = dict(read_data(path, "json"))
    warnings: list[str] = []
    profile = PermissionProfile(
        name="",
        approval=_approval_from_claude(str(data.get("defaultMode", "default"))),
        sandbox="workspace-write",
        include_directories=[str(item) for item in data.get("additionalDirectories", [])],
    )
    permissions = dict(data.get("permissions", {}))
    tool_map = {
        "Read": "read",
        "Edit": "edit",
        "Write": "write",
        "WebFetch": "webfetch",
        "Bash": "shell",
    }

    for decision, attribute_tools, attribute_shell in (
        ("allow", "allow_tools", "allow_shell"),
        ("deny", "deny_tools", "deny_shell"),
        ("ask", "ask_tools", "ask_shell"),
    ):
        for item in permissions.get(decision, []):
            text = str(item)
            if text.startswith("Bash(") and text.endswith(")"):
                getattr(profile, attribute_shell).append(text[5:-1])
                continue
            mapped = tool_map.get(text)
            if mapped:
                getattr(profile, attribute_tools).append(mapped)
            else:
                warnings.append(f"Claude rule '{text}' could not be mapped to gperm.")
    return profile, warnings


def _profile_from_opencode(path: Path) -> tuple[PermissionProfile, list[str]]:
    file_format = "jsonc" if path.suffix == ".jsonc" else "json"
    data = dict(read_data(path, file_format))
    permission = dict(data.get("permission", {}))
    warnings: list[str] = []
    profile = PermissionProfile(name="", approval="default", sandbox="workspace-write")

    edit = permission.get("edit")
    if edit == "allow":
        profile.allow_tools.extend(["edit", "write"])
    elif edit == "deny":
        profile.deny_tools.extend(["edit", "write"])
    elif edit == "ask":
        profile.ask_tools.extend(["edit", "write"])

    webfetch = permission.get("webfetch")
    if webfetch == "allow":
        profile.allow_tools.append("webfetch")
    elif webfetch == "deny":
        profile.deny_tools.append("webfetch")
    elif webfetch == "ask":
        profile.ask_tools.append("webfetch")

    external_directory = permission.get("external_directory")
    if external_directory == "allow":
        profile.trust = True
    elif external_directory == "deny":
        profile.trust = False

    bash = permission.get("bash")
    if isinstance(bash, str):
        if bash == "allow":
            profile.allow_tools.append("shell")
        elif bash == "deny":
            profile.deny_tools.append("shell")
        elif bash == "ask":
            profile.ask_tools.append("shell")
    elif isinstance(bash, dict):
        default_value = bash.get("*")
        if default_value == "allow":
            profile.allow_tools.append("shell")
        elif default_value == "deny":
            profile.deny_tools.append("shell")
        elif default_value == "ask":
            profile.ask_tools.append("shell")

        for key, value in bash.items():
            if key == "*":
                continue
            command = str(key).rstrip("*")
            if value == "allow":
                profile.allow_shell.append(command)
            elif value == "deny":
                profile.deny_shell.append(command)
            elif value == "ask":
                profile.ask_shell.append(command)
            else:
                warnings.append(f"OpenCode bash rule '{key}' has unsupported value '{value}'.")
    elif bash is not None:
        warnings.append("OpenCode bash permission could not be mapped to gperm.")

    return profile, warnings


def _is_claude_global(source: Path, env: dict[str, str]) -> bool:
    return source == (home_dir(env) / ".claude" / "settings.json")


def _is_opencode_global(source: Path, env: dict[str, str]) -> bool:
    root = xdg_config_home(env) / "opencode"
    return source in {root / "opencode.json", root / "opencode.jsonc"}


def _infer_project_root(agent: str, source: Path) -> Path | None:
    if agent == "claude" and source.name == "settings.json" and source.parent.name == ".claude":
        return source.parent.parent
    if agent == "opencode" and source.name in {"opencode.json", "opencode.jsonc"}:
        return source.parent
    return None


def infer_target_path(agent: str, source: Path, env: dict[str, str], explicit_target: Path | None) -> tuple[Path, str]:
    if explicit_target is not None:
        scope = "project" if explicit_target.parent.name == ".gperm" else "global"
        return explicit_target, scope

    if agent == "claude" and _is_claude_global(source, env):
        return xdg_config_home(env) / "gperm" / "config.toml", "global"
    if agent == "opencode" and _is_opencode_global(source, env):
        return xdg_config_home(env) / "gperm" / "config.toml", "global"

    project_root = _infer_project_root(agent, source) or Path.cwd().resolve()
    return project_root / ".gperm" / "config.toml", "project"


def build_import_plan(
    agent: str,
    source_path: Path,
    env: dict[str, str],
    *,
    target_path: Path | None = None,
    profile_name: str | None = None,
) -> ImportPlan:
    normalized = agent.lower()
    if normalized == "claude":
        profile, warnings = _profile_from_claude(source_path)
    elif normalized == "opencode":
        profile, warnings = _profile_from_opencode(source_path)
    else:
        raise ValueError(f"Import is not supported for agent '{agent}'.")

    chosen_profile = profile_name or f"imported-{normalized}"
    profile.name = chosen_profile
    target, scope = infer_target_path(normalized, source_path, env, target_path.resolve() if target_path else None)
    return ImportPlan(
        agent=normalized,
        source_path=source_path,
        target_path=target,
        profile_name=chosen_profile,
        profile=profile,
        scope=scope,
        warnings=warnings,
    )


def merged_import_config(
    target_path: Path,
    plan: ImportPlan,
    *,
    replace_existing_profile: bool,
) -> dict[str, object]:
    base = copy.deepcopy(DEFAULT_CONFIG_DATA)
    if target_path.exists():
        existing = dict(read_data(target_path, "toml"))
        base.update(existing)

    profiles = dict(base.get("profiles", {}))
    if plan.profile_name in profiles and not replace_existing_profile:
        raise FileExistsError(f"Profile '{plan.profile_name}' already exists in {target_path}.")
    profiles[plan.profile_name] = plan.profile.to_dict()
    base["profiles"] = profiles

    if plan.scope == "global":
        agents = dict(base.get("agents", {}))
        agent_entry = dict(agents.get(plan.agent, {}))
        agent_entry["enabled"] = True
        agent_entry["profile"] = plan.profile_name
        agents[plan.agent] = agent_entry
        base["agents"] = agents
    else:
        project = dict(base.get("project", {}))
        project_agents = dict(project.get("agents", {}))
        agent_entry = dict(project_agents.get(plan.agent, {}))
        agent_entry["enabled"] = True
        agent_entry["profile"] = plan.profile_name
        project_agents[plan.agent] = agent_entry
        project["agents"] = project_agents
        base["project"] = project

    return base



from __future__ import annotations

import json
import tomllib
from pathlib import Path
from typing import Any

import json5
import tomlkit


def read_data(path: Path, file_format: str) -> Any:
    if not path.exists():
        return {}
    text = path.read_text(encoding="utf-8")
    if not text.strip():
        return {}
    if file_format in {"json", "jsonc"}:
        return json5.loads(text)
    if file_format == "toml":
        return tomllib.loads(text)
    raise ValueError(f"Unsupported file format: {file_format}")


def write_data(path: Path, file_format: str, data: Any) -> None:
    path.parent.mkdir(parents=True, exist_ok=True)
    if file_format in {"json", "jsonc"}:
        rendered = json.dumps(data, indent=2, ensure_ascii=False, sort_keys=False) + "\n"
    elif file_format == "toml":
        doc = tomlkit.document()
        if isinstance(data, dict):
            for key, value in data.items():
                doc[key] = tomlkit.item(value)
        else:
            raise TypeError("TOML data must be a dictionary")
        rendered = tomlkit.dumps(doc)
    else:
        raise ValueError(f"Unsupported file format: {file_format}")
    path.write_text(rendered, encoding="utf-8")


def write_text(path: Path, content: str) -> None:
    path.parent.mkdir(parents=True, exist_ok=True)
    path.write_text(content, encoding="utf-8")
